% Compute the element load vectors corresponding to nonzero essential boundary
% conditions (settlement of the supports).
% Return an array of them so they may be assembled.
%    Call as
% evs = nz_ebc_loads(feb, geom, u)
%     geom=geometry field
%     u=displacement field
%
function evs = nz_ebc_loads(self, geom, u)
    gcells = get(self.feblock,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Pre-allocate the element matrices
    evs(1:ngcells) = deal(elevec);
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Material
    mat = get(self.feblock, 'mater');
    matstates = get(self, 'matstates');
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = get(gcells(i), 'conn'); % connectivity
        pu = gather(u, conn, 'prescribed_values');
        if norm (pu) ~= 0
            x = gather(geom, conn, 'values', 'noreshape');
            Ke = zeros(dim*nfens); % element stiffness matrix
            % Loop over all integration points
            for j=1:npts_per_gcell
                Nder = Ndermat_param (gcells(i), pc(j,:));
                [Nspder,detJ]=Ndermat_spatial (gcells(i),Nder,x);
                detJ= Jacobian3D(gcells(i), pc(j,:), x'*Nder);
                Rm=material_directions(self,...
                    map_to_xyz(gcells(i),pc(j,:),x),x'*Nder);%loc.b.
                B = Blmat(gcells(i), Nspder, Rm);
                D = tangent_moduli(mat,struct('ms',matstates{i,j}));
                Ke = Ke + B'*D*B * detJ * w(j);
            end
            evs(i) = set(evs(i), 'vec', -Ke*pu);
            evs(i) = set(evs(i), 'eqnums', gather(u, conn, 'eqnums'));
        end
    end
    return;
end
