% Compute the stiffness matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness(feb, geom, u)
%     geom=geometry field
%     u=displacement field
%
function ems = stiffness (self, geom, u)
    gcells = get(self.feblock,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Pre-allocate the element matrices
    ems(1:ngcells) = deal(elemat);
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Material
    mat = get(self.feblock, 'mater');
    matstates = get(self, 'matstates');
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = get(gcells(i), 'conn'); % connectivity
        x = gather(geom, conn, 'values', 'noreshape'); % coord
        Ke = zeros(dim*nfens); % element stiffness matrix
        % Loop over all integration points
        for j=1:npts_per_gcell
            Nder = Ndermat_param (gcells(i), pc(j,:));
            [Nspder,detJ]=Ndermat_spatial (gcells(i),Nder,x);
            detJ= Jacobian3D(gcells(i), pc(j,:), x'*Nder);
            Rm=material_directions(self,...
                map_to_xyz(gcells(i),pc(j,:),x),x'*Nder);%loc.b.            
            B = Blmat(gcells(i), Nspder, Rm);
            D = tangent_moduli(mat,struct('ms',matstates{i,j}));
            Ke = Ke + B'*D*B * detJ * w(j);
        end
        ems(i) = set(ems(i), 'mat', Ke);
        ems(i) = set(ems(i), 'eqnums', gather(u,conn,'eqnums'));
    end
    return;
end