% Constructor.
% Class representing finite element nodes, in one dimension,
% two dimensions, or three dimensions.
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options:
% id = identifier of node, xyz= coordinates as a one-dimensional,
% two-dimensional, or three-dimensional array.
%
function self = fenode (varargin)
    class_name='fenode';
    parent = classbase;
    if (nargin == 0)
        self.id = 0;
        self.xyz = [];
        self = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            self = arg;
            return;
        else
            options = varargin{1};
            self.id = options.id;
            xyz = options.xyz;
            [m,n]=size(xyz);
            self.xyz = xyz;
            self = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end


