% Constructor
% class represents a four-node quadrilateral geometric cell, which encloses some part
% of the computational domain.
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options: same as gcell.
% and thickness = thickness of the element
%
%   Example: gcell_Q4(struct ('id',13,'conn',[1 3 8 4], 'thickness',0.2))  creates a cell
%      that connects nodes 1 3 8 4, with thickness 0.2, and gives it the `name' 13.
%
function retobj = gcell_Q4 (varargin)
    class_name='gcell_Q4';
    nfens=4;
    if (nargin == 0)
        parent=gcell;
        retobj = class(struct([]),class_name, parent);
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options=varargin{1}; options.nfens=nfens;
            parent=gcell(options);
            retobj = class(struct([]),class_name, parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end