% Constructor
% Class represents a four-node tetrahedron geometric cell, which encloses some part
% of the computational domain.
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options: same as gcell.
%
%   Example: gcell_t4(struct ('id',13,'conn',[1 8 4 3]))  creates a cell
%      that connects nodes 1 8 4 3, and gives it the `name' 13.
%
function retobj = gcell_t4 (varargin)
    class_name='gcell_t4';
    nfens=4;
    if (nargin == 0)
        options.nfens=nfens;
        parent=gcell_3_manifold(options);
        retobj = class(struct([]),class_name, parent);
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options=varargin{1}; options.nfens=nfens;
            parent=gcell_3_manifold(options);
            retobj = class(struct([]),class_name, parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end